package mcmultipart.microblock;

import java.util.Map.Entry;

import com.google.common.base.Function;
import com.google.common.base.Joiner;
import com.google.common.collect.Iterables;

import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.IBlockState;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.Tuple;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraftforge.fml.relauncher.ReflectionHelper;

import mcmultipart.microblock.IMicroMaterial.IDelegatedMicroMaterial;

/**
 * A simple implementation of {@link IMicroMaterial} that's defined based on an {@link IBlockState}.
 */
public class BlockMicroMaterial implements IMicroMaterial {

    private static final Joiner COMMA_JOINER = Joiner.on(',');
    @SuppressWarnings("rawtypes")
    private static final Function<Entry<IProperty<?>, Comparable<?>>, String> MAP_ENTRY_TO_STRING = new Function<Entry<IProperty<?>, Comparable<?>>, String>() {

        @SuppressWarnings("unchecked")
        @Override
        public String apply(Entry<IProperty<?>, Comparable<?>> entry) {

            if (entry == null) {
                return "<NULL>";
            } else {
                IProperty iproperty = entry.getKey();
                return iproperty.func_177701_a() + "=" + iproperty.func_177702_a(entry.getValue());
            }
        }
    };

    private final IBlockState blockState;
    private final float hardness;
    private final String name;

    public BlockMicroMaterial(IBlockState blockState) {

        this(blockState, ((Float) ReflectionHelper.getPrivateValue(Block.class, blockState.func_177230_c(), "blockHardness", "field_149782_v"))
                .floatValue());
    }

    public BlockMicroMaterial(IBlockState blockState, float hardness) {

        this.blockState = blockState;
        this.hardness = hardness;
        this.name = genName();
    }

    private BlockMicroMaterial(BlockMicroMaterial material) {

        this.blockState = material.blockState;
        this.hardness = material.hardness;
        this.name = material.name;
    }

    private final String genName() {

        StringBuilder stringbuilder = new StringBuilder();
        stringbuilder.append(Block.field_149771_c.func_177774_c(blockState.func_177230_c()));
        if (!blockState.func_177228_b().isEmpty()) {
            stringbuilder.append("[");
            COMMA_JOINER.appendTo(stringbuilder, Iterables.transform(blockState.func_177228_b().entrySet(), MAP_ENTRY_TO_STRING));
            stringbuilder.append("]");
        }
        return stringbuilder.toString();
    }

    @Override
    public String getName() {

        return name;
    }

    @Override
    public String getLocalizedName() {

        return getItem().func_82833_r();
    }

    @Override
    public boolean isSolid() {

        return blockState.func_185917_h();
    }

    @SuppressWarnings("deprecation")
    @Override
    public int getLightValue() {

        return blockState.func_185906_d();
    }

    @Override
    public float getHardness() {

        return hardness;
    }

    @Override
    public int getSawStrength() {

        return blockState.func_177230_c().getHarvestLevel(blockState);
    }

    @Override
    public ItemStack getItem() {

        return new ItemStack(blockState.func_177230_c(), 1, blockState.func_177230_c().func_176201_c(blockState));
    }

    @Override
    public SoundType getSound() {

        return blockState.func_177230_c().func_185467_w();
    }

    @Override
    public boolean canRenderInLayer(BlockRenderLayer layer) {

        return blockState.func_177230_c().canRenderInLayer(getDefaultMaterialState(), layer);
    }

    @Override
    public IBlockState getDefaultMaterialState() {

        return blockState;
    }

    @Override
    public IBlockState getMaterialState(IBlockAccess world, BlockPos pos, IMicroblock microblock) {

        return blockState;
    }

    public DelegatedBlockMicroMaterial withDelegate(Function<Tuple<IMicroblock, Boolean>, MicroblockDelegate> delegateFactory) {

        return new DelegatedBlockMicroMaterial(this, delegateFactory);
    }

    public static class DelegatedBlockMicroMaterial extends BlockMicroMaterial implements IDelegatedMicroMaterial {

        private final Function<Tuple<IMicroblock, Boolean>, MicroblockDelegate> delegateFactory;

        private DelegatedBlockMicroMaterial(BlockMicroMaterial material,
                Function<Tuple<IMicroblock, Boolean>, MicroblockDelegate> delegateFactory) {

            super(material);
            this.delegateFactory = delegateFactory;
        }

        @Override
        public MicroblockDelegate provideDelegate(IMicroblock microblock, boolean isRemote) {

            return delegateFactory.apply(new Tuple<IMicroblock, Boolean>(microblock, isRemote));
        }
    }

}
